@@ -14,7 +14,6 @@
 import collections
 import logging
 
-import django
 from django.conf import settings
 from django.contrib.auth import authenticate  # noqa
 from django.contrib.auth import forms as django_auth_forms
@@ -61,7 +60,9 @@ class Login(django_auth_forms.AuthenticationForm):
         if getattr(settings,
                    'OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT',
                    False):
+            last_domain = self.request.COOKIES.get('login_domain', None)
             self.fields['domain'] = forms.CharField(
+                initial=last_domain,
                 label=_("Domain"),
                 required=True,
                 widget=forms.TextInput(attrs={"autofocus": "autofocus"}))
@@ -84,8 +85,7 @@ class Login(django_auth_forms.AuthenticationForm):
                 choices=getattr(settings, 'WEBSSO_CHOICES', ()),
                 required=False,
                 initial=initial)
-            # move auth_type to the top of the list
-            fields_ordering.pop(-1)
+            # add auth_type to the top of the list
             fields_ordering.insert(0, 'auth_type')
 
         # websso is enabled, but keystone version is not supported
@@ -93,13 +93,8 @@ class Login(django_auth_forms.AuthenticationForm):
             msg = ("Websso is enabled but horizon is not configured to work " +
                    "with keystone version 3 or above.")
             LOG.warning(msg)
-        # Starting from 1.7 Django uses OrderedDict for fields and keyOrder
-        # no longer works for it
-        if django.VERSION >= (1, 7):
-            self.fields = collections.OrderedDict(
-                (key, self.fields[key]) for key in fields_ordering)
-        else:
-            self.fields.keyOrder = fields_ordering
+        self.fields = collections.OrderedDict(
+            (key, self.fields[key]) for key in fields_ordering)
 
     @staticmethod
     def get_region_choices():
@@ -129,12 +124,18 @@ class Login(django_auth_forms.AuthenticationForm):
                                            password=password,
                                            user_domain_name=domain,
                                            auth_url=region)
-            msg = 'Login successful for user "%(username)s".' % \
-                {'username': username}
+            msg = 'Login successful for user "%(username)s", remote address '\
+                '%(remote_ip)s.' % {
+                    'username': username,
+                    'remote_ip': utils.get_client_ip(self.request)
+                }
             LOG.info(msg)
         except exceptions.KeystoneAuthException as exc:
-            msg = 'Login failed for user "%(username)s".' % \
-                {'username': username}
+            msg = 'Login failed for user "%(username)s", remote address '\
+                '%(remote_ip)s.' % {
+                    'username': username,
+                    'remote_ip': utils.get_client_ip(self.request)
+                }
             LOG.warning(msg)
             raise forms.ValidationError(exc)
         if hasattr(self, 'check_for_test_cookie'):  # Dropped in django 1.7
